package com.fsck.k9.helper

import com.fsck.k9.mail.Address

/**
 * Helper class to access the contacts stored on the device.
 */
class Contacts {
    /**
     * Mark contacts with the provided email addresses as contacted.
     */
    fun markAsContacted(addresses: Array<Address?>?) {
        // TODO: Keep track of this information in a local database. Then use this information when sorting contacts for
        // auto-completion.
    }
}
