package app.k9mail.core.ui.compose.theme2.default

import androidx.compose.ui.unit.dp
import app.k9mail.core.ui.compose.theme2.ThemeSizes

val defaultThemeSizes = ThemeSizes(
    smaller = 8.dp,
    small = 16.dp,
    medium = 32.dp,
    large = 64.dp,
    larger = 128.dp,
    huge = 256.dp,
    huger = 384.dp,

    iconSmall = 16.dp,
    icon = 24.dp,
    iconLarge = 32.dp,
    iconAvatar = 48.dp,

    topBarHeight = 64.dp,
    bottomBarHeight = 80.dp,
    bottomBarHeightWithFab = 72.dp,
)
