object ThunderbirdPlugins {

    object App {
        const val android = "thunderbird.app.android"
        const val androidCompose = "thunderbird.app.android.compose"

        const val jvm = "thunderbird.app.jvm"
    }
    object Library {
        const val android = "thunderbird.library.android"
        const val androidCompose = "thunderbird.library.android.compose"

        const val jvm = "thunderbird.library.jvm"
    }
}
