package com.fsck.k9.widget.list

import app.k9mail.feature.widget.message.list.BaseMessageListWidgetProvider

/**
 * IMPORTANT: The fully qualified name for this class must be
 *            `com.fsck.k9.widget.list.MessageListWidgetProvider`.
 *             Otherwise widgets created with older versions of the app using a different name
 *             will stop working or disappear.
 */
class MessageListWidgetProvider : BaseMessageListWidgetProvider()
